@@ -75,7 +75,7 @@
 #define WARN_TIMEOUT	10		/* warn about this every N seconds */
 
 static const char *applet;
-static char *service, *runlevel, *ibsave, *prefix;;
+static char *service, *runlevel, *ibsave, *prefix;
 static RC_DEPTREE *deptree;
 static RC_STRINGLIST *applet_list, *services, *tmplist;
 static RC_STRINGLIST *restart_services, *need_services, *use_services;
@@ -288,6 +288,13 @@ cleanup(void)
 #endif
 }
 
+/* Buffer and lock all output messages so that we get readable content */
+/* FIXME: Use a dynamic lock file that contains the tty/pts as well.
+ * For example openrc-pts8.lock or openrc-tty1.lock.
+ * Using a static lock file makes no sense, esp. in multi-user environments.
+ * Why don't we use (f)printf, as it is thread-safe through POSIX already?
+ * Bug: 360013
+ */
 static int
 write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 {
@@ -297,14 +304,20 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 	ssize_t ret = 0;
 	int fd = fileno(stdout), lock_fd = -1;
 
-	/* Spin until we lock the prefix */
-	for (;;) {
-		lock_fd = open(PREFIX_LOCK, O_WRONLY | O_CREAT, 0664);
-		if (lock_fd != -1)
-			if (flock(lock_fd, LOCK_EX) == 0)
-				break;
-		close(lock_fd);
+	/*
+	 * Lock the prefix.
+	 * open() may fail here when running as user, as RC_SVCDIR may not be writable.
+	 */
+	lock_fd = open(PREFIX_LOCK, O_WRONLY | O_CREAT, 0664);
+
+	if (lock_fd != -1) {
+		if (flock(lock_fd, LOCK_EX) != 0)
+			eerror("flock() failed: %s", strerror(errno));
 	}
+#ifdef RC_DEBUG
+	else
+		ewarn("Couldn't open the prefix lock, please make sure you have enough permissions");
+#endif
 
 	for (i = 0; i < bytes; i++) {
 		/* We don't prefix eend calls (cursor up) */
@@ -332,6 +345,7 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 
 	/* Release the lock */
 	close(lock_fd);
+
 	return ret;
 }
 
@@ -1079,12 +1093,13 @@ service_plugable(void)
 }
 
 #include "_usage.h"
-#define getoptstring "dDsvl:Z" getoptstring_COMMON
+#define getoptstring "dDsSvl:Z" getoptstring_COMMON
 #define extraopts "stop | start | restart | describe | zap"
 static const struct option longopts[] = {
 	{ "debug",      0, NULL, 'd'},
 	{ "dry-run",    0, NULL, 'Z'},
 	{ "ifstarted",  0, NULL, 's'},
+	{ "ifstopped",  0, NULL, 'S'},
 	{ "nodeps",     0, NULL, 'D'},
 	{ "lockfd",     1, NULL, 'l'},
 	longopts_COMMON
@@ -1093,6 +1108,7 @@ static const char *const longopts_help[] = {
 	"set xtrace when running the script",
 	"show what would be done",
 	"only run commands when started",
+	"only run commands when stopped",
 	"ignore dependencies",
 	"fd of the exclusive lock from rc",
 	longopts_help_COMMON
@@ -1235,6 +1251,10 @@ runscript(int argc, char **argv)
 			if (!(rc_service_state(service) & RC_SERVICE_STARTED))
 				exit(EXIT_FAILURE);
 			break;
+		case 'S':
+			if (!(rc_service_state(service) & RC_SERVICE_STOPPED))
+				exit(EXIT_FAILURE);
+			break;
 		case 'D':
 			deps = false;
 			break;
